/***********************************************************************/
/*                                                                     */
/*                              Pio (C++)                              */
/*                                                                     */
/*                    Polyglot Input/Output Library                    */
/*                                                                     */
/*                  projet Estime, INRIA-Rocquencourt                  */
/*                                                                     */
/*  Copyright 2004-2011 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Estelle Marchand <Estelle.Marchand@inria.fr>                       */
/*                                                                     */
/***********************************************************************/

/********************************************************************************/
/*                                                                              */
/*                                  Pio (C++)                                   */
/*                                                                              */
/*                        Polyglot Input/Output Library                         */
/*                                                                              */
/*                      projet Estime, INRIA-Rocquencourt                       */
/*                                                                              */
/*  Copyright 2004-2011 INRIA.                                                  */
/*  Institut National de Recherche en Informatique et en Automatique.           */
/*  All rights reserved. This file is distributed only by permission.           */
/*                                                                              */
/*  Estelle Marchand <Estelle.Marchand@inria.fr>                                */
/*                                                                              */
/********************************************************************************/

/***********************************************************************/
/*                                                                     */
/*                              PIO (C++)                              */
/*                                                                     */
/*                    Polyglot Input/Output Library                    */
/*                                                                     */
/*                  projet Estime, INRIA-Rocquencourt                  */
/*                                                                     */
/*  Copyright 2005-2007 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Estelle Marchand <Estelle.Marchand@inria.fr>                       */
/*                                                                     */
/***********************************************************************/

/* $Id: send_typed_value.cpp,v 1.3 2011-02-23 17:43:29 fclement Exp $ */

#include "send_typed_value.hpp"

using namespace std;

namespace io{
  
  //for the generic function:
  

  template<class dataType> void send_typed_value(ostream& ob, const dataType &  p){
    switch( p.type() ){
    case INT: send(ob,p.to_int()); break;
    case STRING: send(ob,p.to_string()); break;
    case BOOLEAN: send(ob,p.to_boolean()); break;
    case FLOAT: send(ob,p.to_float()); break;
    case FLOAT_TRIPLE: send(ob,p.to_float_triple()); break;
    case FLOAT_COUPLE: send(ob,p.to_float_couple()); break;
    case INT_TRIPLE: send(ob,p.to_int_triple()); break;
    case INT_COUPLE: send(ob,p.to_int_couple()); break;
    case BOOLEAN_TRIPLE: send(ob,p.to_boolean_triple()); break;
    case BOOLEAN_COUPLE: send(ob,p.to_boolean_couple()); break;
    default: {cerr<<"send_typed_value: error: unknown Pio type of object"<<endl;ElemError t;throw(t);} 
    }
  }
  
  void send_typed_value(ostream& ob, Matrix_type1 &  p){
    int mm=p.n_rows(),nn=p.n_cols();
    switch( p.type() ){
    case INT: {cerr<<"error: cannot send a non float matrix"<<endl;exit (-1);}//send(ob,mm,nn,p.i.data());
    case BOOLEAN: {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);}//send(ob,mm,nn,p.b.data());
    case STRING: {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);}//send(ob,mm,nn,p.s.data());
    case FLOAT: {
      if(p.layout()=='C')
        send_row_float_matrix(ob,mm,nn,p.d.data());
      else
        send_column_to_column_float_matrix(ob,mm,nn,p.d.data());
      break;
    }
    case FLOAT_TRIPLE: {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);} //send(ob,mm,nn,p.ft.data());
    case FLOAT_COUPLE: {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);} //send(ob,mm,nn,p.fc.data());
    case INT_TRIPLE:  {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);}//send(ob,mm,nn,p.it.data());
    case INT_COUPLE: {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);}//send(ob,mm,nn,p.ic.data());
    case BOOLEAN_TRIPLE:  {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);}//send(ob,mm,nn,p.it.data());
    case BOOLEAN_COUPLE: {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);}//send(ob,mm,nn,p.ic.data());
    default: {cerr<<"send_typed_value: error: unknown Pio type of object"<<endl;ElemError t;throw(t);}
    }
  }
  
  void send_typed_value(ostream& ob,Matrix_type2 &  p){
    int mm=p.n_rows(),nn=p.n_cols();
    switch( p.type() ){
    case INT:  {cerr<<"error: cannot send a non float matrix"<<endl;exit (-1);}//send(ob,mm,nn,p.i.data());
    case BOOLEAN: {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);}//send(ob,mm,nn,p.b.data());
    case STRING: {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);}//send(ob,mm,nn,p.s.data());
    case FLOAT: send_row_float_matrix(ob,mm,nn,p.d.data());break;
    case FLOAT_TRIPLE: {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);} //send(ob,mm,nn,p.ft.data());
    case FLOAT_COUPLE: {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);} //send(ob,mm,nn,p.fc.data());
    case INT_TRIPLE: {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);}//send(ob,mm,nn,p.it.data());
    case INT_COUPLE: {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);}//send(ob,mm,nn,p.ic.data());
    case BOOLEAN_TRIPLE:  {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);}//send(ob,mm,nn,p.it.data());
    case BOOLEAN_COUPLE: {cerr<<"error: cannot send a non float matrix"<<endl;exit(-1);}//send(ob,mm,nn,p.ic.data());
    default: {cerr<<"send_typed_value: error: unknown Pio type of object"<<endl;ElemError t;throw(t);}
    }
  }
  
  void send(ostream& ob, Typed_value & p){
    switch( p.type() ){
    case LEXEM: send_typed_value(ob,p.to_Lexem());break;
    case VECTOR: send_typed_value(ob,p.to_Vector_type());break;
    case MATRIX1: send_typed_value(ob,p.to_Matrix_type1());break;
    case MATRIX2: send_typed_value(ob,p.to_Matrix_type2());break;
    default: {cerr<<"send: error: unknown type of object"<<endl;ElemError t;throw(t);}
    }
  }
  
  void send_typed_value(ostream& ob, Typed_value & p){send(ob,p);}
  
  //  void send(ostream& ob,OTyped_value & p){
  //    send(ob,p.value());
  //   }
  
  //   void send(ostream& ob,OMatrix_type1 & p){
  //    send_row_float_matrix(ob,p.n_rows(),p.n_columns(),p.value_());
  //   }
  
  //   void send(ostream& ob,OMatrix_type2 & p){
  //    send_row_float_matrix(ob,p.n_rows(),p.n_columns(),p.value_());
  //   }
  
  

}
